#!/usr/bin/perl
#
# Simple web management interface for home Linux server
#

use strict;
use CGI;

#  Associate module names with handling functions
my %module_handler = (
	'mp3'		=> \&mp3_handler,
	'volume'	=> \&volume_handler,
	'server'	=> \&server_handler
);

#-----------------------------------------------------------
# Main
#-----------------------------------------------------------

# instantiate a new CGI object
my $cgi = new CGI;

# Read user submitted arguments
my $user_module = $cgi->param('module');
my $user_action = $cgi->param('action');

# Call appropriate module handler based on user input
my $result_string;
if (defined $module_handler{$user_module}) {
	$result_string = $module_handler{$user_module}->($user_action);
} else {
	$result_string = "Invalid module '$user_module'";
}

# Print output, with module result in <h1> tags
print
	$cgi->header .
	$cgi->start_html($result_string) .
	$cgi->h1($result_string) .
	$cgi->end_html;
exit (0);


#-----------------------------------------------------------
# Handlers
#-----------------------------------------------------------

# Start/stop MP3 playback (using MPC/MPD)
sub mp3_handler() {
	my($action) = @_;
	my($cmd, $out);
	if ($action eq 'start') { $cmd = "/usr/bin/mpc play"; }
	elsif ($action eq 'stop') { $cmd = "/usr/bin/mpc stop"; }
	else { return "mp3_handler: bad action '$action'"; }
	$out = `$cmd 2>&1`;
	return "MP3 $action: $out";
}

# Increase/decrease/mute/unmute the playback volume of PCM audio device
sub volume_handler() {
	my($action) = @_;
	my($change, $out);

	if ($action eq 'up') { $change = "10%+"; }
	elsif ($action eq 'down') { $change = "10%-"; }
	elsif ($action eq 'mute') { $change = "mute"; }
	elsif ($action eq 'unmute') { $change = "unmute"; }
	else { return "volume_handler: bad action '$action'"; }
	my $cmd = "amixer cset iface=MIXER,name='PCM Playback Volume' $change";
	chop($out = `$cmd 2>&1`);
	return "Volume $change: $out";
}

# Perform misc server functions
sub server_handler() {
	my($action) = @_;
	my($cmd, $out);
	if ($action eq 'uptime') {
		# Send back output from Linux uptime command
		$cmd = "/usr/bin/uptime";
		chop($out = `$cmd 2>&1`);
	} elsif ($action eq 'diskusage') {
		# Run df, get Filesystem Used/Total values (in Mb) for data partitions
		my $r;
		foreach $r (`df -BM | grep /dev/sd`) {
			chop($r);
			my @parts = split(/\s+/, $r);
			$out .= "$parts[0]: $parts[2]/$parts[1] \n";
		}
	} elsif ($action eq 'shutdown') {
		# Shutdown the server
		$cmd = "/sbin/shutdown -t1 -a -h now";
		$out = `$cmd 2>&1`;
	} else {
		return "server_handler: bad action '$action'";
	}
	return $out;
}

